SystemOrganization addCategory: #'Idiom-Exercise'!

TestCase subclass: #IdiomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!IdiomTest methodsFor: 'testing' stamp: 'PabloRago 4/22/2013 21:35'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds

	| customerBook |
	
	customerBook := CustomerBook  new.
	
	customerBook  assert:  
		[customerBook addCustomerNamed: 'John Lennon'.]
		isExecutedIn: 50

	
! !

!IdiomTest methodsFor: 'testing' stamp: 'PabloRago 4/22/2013 21:35'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook johnLennon |
	
	customerBook := CustomerBook  new.
	johnLennon := 'John Lennon'.
	
	customerBook addCustomerNamed: johnLennon.
	  
	customerBook assert:
		[customerBook removeCustomerNamed: johnLennon.]
		isExecutedIn: 100.
	
	
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'PabloRago 4/22/2013 21:47'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.

	customerBook fails:  [ customerBook addCustomerNamed: ''.] 
		onSignal: Error 
		do: [:anError|
			self assert: (anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage).
			self assert: (customerBook isEmpty ).
		].! !

!IdiomTest methodsFor: 'testing' stamp: 'PabloRago 4/22/2013 21:47'!
test04CanNotRemoveAndInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.

	customerBook fails: [ customerBook removeCustomerNamed: 'Paul McCartney'.] 
			onSignal: NotFound 
			do: [ :anError |
				self assert: (customerBook numberOfCustomers = 1).
				self assert: (customerBook includesCustomerNamed: johnLennon). ]! !

Object subclass: #CustomerBook
	instanceVariableNames: 'customers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:57'!
customerAlreadyExistsErrorMessage

	^'Customer already exists'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:53'!
customerCanNotBeEmptyErrorMessage

	^'Customer name cannot be empty'! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:42'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	(customers includes: aName) ifTrue: [ self signalCustomerAlreadyExists ].
	
	customers add: aName ! !

!CustomerBook methodsFor: 'testing' stamp: 'PabloRago 4/22/2013 20:03'!
assert: aBlock isExecutedIn: aTime 
	""

	| millisecondsBeforeRunning millisecondsAfterRunning |
	
	millisecondsBeforeRunning := Time millisecondClockValue.
	[ aBlock value]. 	
	millisecondsAfterRunning := Time millisecondClockValue.
	
	self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) < aTime! !

!CustomerBook methodsFor: 'testing' stamp: 'PabloRago 4/22/2013 21:46'!
fails: aBlock onSignal: signal do: anotherBlock
	""

	[ aBlock value.
	self fail ]
		on: signal 
		do: [ :anError | 
			anotherBlock value: anError].! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:56'!
includesCustomerNamed: aName

	^customers includes: aName ! !

!CustomerBook methodsFor: 'initialization' stamp: 'HernanWilkinson 7/6/2011 17:42'!
initialize

	super initialize.
	customers := OrderedCollection new! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:48'!
isEmpty
	
	^customers isEmpty  ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:56'!
numberOfCustomers
	
	^customers size! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:44'!
removeCustomerNamed: aName
 
	customers remove: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:52'!
signalCustomerAlreadyExists 

	self error: self class customerAlreadyExistsErrorMessage! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:51'!
signalCustomerNameCannotBeEmpty 

	self error: self class customerCanNotBeEmptyErrorMessage ! !
